#!/usr/bin/python
#------------------------------------------------------------------------------
# Copyright (c) 2007-2014 Perforce Software, Inc.  Provided for use as defined in
# the Perforce Consulting Services Agreement.
#------------------------------------------------------------------------------

"""
This script creates groups on the server based on the entries in a local file called groups.txt
The groups.txt file contains:

group,groupname1
username
username
etc
group,groupname2
username
username
etc

Run:
  python creategroups [instance]

Instance defaults to 1 if not given.
"""

import sys
import os
import platform
import re
from subprocess import *
import shutil
import ConfigParser

if len(sys.argv) > 1:
  P4INSTANCE = str(sys.argv[1])
else:
  P4INSTANCE = '1'

config = ConfigParser.RawConfigParser()
config.read('maintenance.cfg')

##########################################################################
#####                                                                #####
#####  CONFIGURATION VARIABLES: Modify in maintenance.cfg as needed. #####
#####                                                                #####

server = (config.get(P4INSTANCE, 'server'))
p4user = (config.get(P4INSTANCE, 'p4user'))
passwd = (config.get(P4INSTANCE, 'passwd'))
weeks = int(config.get(P4INSTANCE, 'weeks'))

if platform.system() == "Windows": 
  p4="p4.exe -p %s -u %s" % (server, p4user)
else:
  p4="/p4/1/bin/p4_1  -p %s -u %s" % (server, p4user)

os.system('echo %s| %s login' % (passwd, p4))

def log(msglevel="DEBUG", message=""):
    if msglevel == "ERROR":
        print(message)
        sys.exit(1)
    else:
        print(message)

def runp4cmd(cmd):
    try:
        pipe = Popen(p4 + cmd, shell=True, stdin=PIPE, stdout=PIPE, universal_newlines=True)
        stdout, stderr = pipe.communicate()
        log("DEBUG", stdout)
        if pipe.returncode != 0:
            log("ERROR", "%s%s generated the following error: %s" % (p4, cmd, stderr))
        else:
            return stdout
    except OSError as err:
        log("ERROR", "Execution failed: %s" % (err))

def main(): 
    groupsfile = open("groups.txt", "r")
    groupfile = open("group.txt", "w")
    count = 0
    
    for line in groupsfile.readlines():
        try:
            group = (re.match("^group,(.*)", line).groups())
            if count == 0:
                groupfile.write('Group:\t%s\nUsers:\n' % group[0])
                count = 1
            else:
                groupfile.close()
                cmd = 'group -i < group.txt'
                runp4cmd(cmd)
                os.remove('group.txt')
                groupfile = open("group.txt", "w")
                groupfile.write('Group:\t%s\nUsers:\n' % group[0])
        except:
            groupfile.write('\t%s' % line)

    groupfile.close()
    cmd = 'group -i < group.txt'
    runp4cmd(cmd)
    os.remove('group.txt')

if __name__ == "__main__":
    main()
